import numpy as np
import pandas as pd
from sklearn.preprocessing import LabelEncoder, OneHotEncoder
import json
import pytemperature as pt
from pymongo import MongoClient
from keras.layers import Input, Embedding, Flatten, Dropout, Dense, Concatenate, Reshape, LSTM
from keras.models import Model, Sequential
import random

# pprint library is used to make the output look more pretty
# from pprint import pprint

restaurant_ratings = []
restaurant_categories = []
restaurant_prices = []
temperatures = []
distances = []
busynesses = []
times_in_hour = []
dates = [] # just use to track training
time_stamps = [] # just use to track training
actions = []

client = MongoClient('confid','port')
db = client['name']
db.authenticate('user','password')


def train_next_acion_for_user(id):
    connections = db.connections2.find({'user_id':id}).sort('_id',1)
    le_category = LabelEncoder()
    ohe_category = OneHotEncoder()

    for connection in connections:
        #print connection['date'], ' and ',connection['time']
        dates.append(connection['date'])
        time_stamps.append(connection['time'])
        #print(connection['restaurant_rating'])
        #print(type(connection['restaurant_rating']))
        restaurant_ratings.append(int(round(float(connection['restaurant_rating']))))
        restaurant_prices.append(connection['restaurant_price'])
        if connection['restaurant_category'] != None:
            restaurant_categories.append(connection['restaurant_category'].encode('utf-8'))
        else:
            restaurant_categories.append(np.nan)
        temperatures.append(int(pt.k2f(connection['temperature']['temp'])))
        if (connection['distance'] != {} and 'hour' not in connection['distance']['duration']['text']):
            minutes = int(connection['distance']['duration']['text'].replace(" min","").replace("s",""))
            distances.append(minutes)
        else:
            distances.append(10)
        if int(connection['busyness']) == 0:
            busynesses.append(20)
        else:
            busynesses.append(int(connection['busyness']))
        time_data = connection['time'].split("-")
        if time_data[3] == 'pm' and int(time_data[0]) < 12:
            times_in_hour.append(int(time_data[0]) + 12)
        elif time_data[3] == 'am' and int(time_data[0]) == 12:
            times_in_hour.append(24)
        else:
            times_in_hour.append(int(time_data[0]))

        actions.append(int(connection['action']))
    
    data = {
        'restaurant_categories': restaurant_categories,
        'restaurant_prices': restaurant_prices,
        'restaurant_ratings': restaurant_ratings,
        'times_in_hour': times_in_hour,
        'distances': distances,
        'busynesses': busynesses,
        'temperatures': temperatures, 
        'actions': actions
    }

    #sample random sequences from data
    sequence_index_list = random.sample(xrange(0, len(restaurant_ratings) - 21), 600)
    #sequence_matrix = np.empty(shape=[300, 34])
    input_sequences = []
    output_actions = []
    for sequence_index in sequence_index_list:
        sequence = []
        for i in range (20):
            sequence.append(restaurant_prices[sequence_index + i])
            sequence.append(restaurant_ratings[sequence_index + i])
            sequence.append(times_in_hour[sequence_index + i])
            sequence.append(distances[sequence_index +i])
            sequence.append(busynesses[sequence_index +i])
            sequence.append(temperatures[sequence_index +i])
            #print dates[sequence_index+i], ' and ', time_stamps[sequence_index + i]
            if i < 19:
                sequence.extend(convert_to_one_hot_encoding(actions[sequence_index +i]))
            else:
                output_actions.append(convert_to_one_hot_encoding(actions[sequence_index +i]))
        input_sequences.append(sequence)
    sequence_X = np.array(input_sequences).reshape(600,196,1)
    sequence_Y = np.array(output_actions)
    print sequence_Y.shape

    embedding_vecor_length = 30
    model = Sequential()
    model.add(LSTM(20, input_shape=(196, 1)))
    model.add(Dense(20, activation='sigmoid'))
    model.add(Dense(15, activation='sigmoid'))
    model.add(Dense(10, activation='sigmoid'))
    model.add(Dense(4, activation='sigmoid'))
    model.compile(loss='binary_crossentropy', optimizer='adam', metrics=['accuracy'])
    print(model.summary())
    model.fit(sequence_X, sequence_Y, epochs=5, batch_size=20)
    model.save(str(id)+'.h5')

def convert_to_one_hot_encoding(action):
    bucket = [0] * 4
    bucket[action] = 1
    return bucket

'''
