import numpy as np
import pandas as pd
## SET BACKEND
import matplotlib as mpl
mpl.use('TkAgg')
import matplotlib.pyplot as plt
import os
import warnings

from keras.layers import Input, Embedding, Flatten, Dropout, Dense, Concatenate
from keras.models import Model
os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2'

dataset = pd.read_csv('user_business_rating_with_category2.dat', delim_whitespace= True, header=0)
print (dataset.head())
print (dataset.shape)
from sklearn.model_selection import train_test_split
train, test = train_test_split(dataset, test_size=0.2, random_state=42)
print (train.head())
print (test.head())
n_users = len(dataset.user_id.unique())
print(n_users)
n_businesses = len(dataset.business_id.unique())
print(n_businesses)
print np.sort(dataset.business_id.unique())
n_categories = len(dataset.category.unique())
print np.sort(dataset.category.unique())
#print (dataset.category.unique())
embedding_size = 10

business_input = Input(shape=[1], name="business")
business_embedding = Embedding(32516+1, embedding_size, name="business-embedding")(business_input)
business_vec = Flatten(name="flatten-businesses")(business_embedding)

user_input = Input(shape=[1], name="user")
user_embedding = Embedding(n_users+1, embedding_size, name="user-embedding")(user_input)
user_vec = Flatten(name="flatten-users")(user_embedding)

category_input = Input(shape=[1], name="category")
category_embedding = Embedding(69+1, embedding_size, name="category-embedding")(category_input)
category_vec = Flatten(name="flatten-categories")(category_embedding)

input_vecs = Concatenate()([business_vec, user_vec, category_vec])

a = Dense(128, activation='relu')(input_vecs)
b = Dropout(0.2)(a)
c = Dense(64)(b)
d = Dropout(0.2)(c)
e = Dense(32)(d)
f = Dropout(0.2)(e)
g = Dense(1)(f)


model = Model([user_input, business_input, category_input], g)
model.compile('adam', 'mean_squared_error', metrics=['accuracy'])

from keras.models import load_model

if os.path.exists('regression_model_new.h5'):
    model = load_model('regression_model_new.h5')
else:
    history = model.fit([train.user_id, train.business_id, train.category], train.rating, epochs=5, verbose=1, validation_data=([test.user_id, test.business_id, test.category], test.rating))
    model.save('regression_model_new.h5')
    loss = plt.plot(history.history['loss'])
    val_loss = plt.plot(history.history['val_loss'])
    plt.plot(loss)
    plt.plot(val_loss)
    plt.xlabel("Epochs")
    plt.ylabel("Training Error")
    plt.legend(['loss', 'val_loss'])
    plt.show()


